//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
/*+==========================================================================
  File:      sockget.cs

  Summary:   Demonstrates asyncronous sockets using the Socket class in
             the URT Frameworks SDK. Sends a "GET" to a specified web server
             then receives the default web page.

  Classes:   AsyncData, SockGet

  Functions: Main, DoGet, DoSend, DoReceive, ConnectCallback, SendCallback,
             ReceiveCallback

----------------------------------------------------------------------------  

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.
==========================================================================+*/

using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

public class AsyncData
{
	public Socket socket;
	public byte[] RecBytes = new byte[256];
}

public class SockGet
{

	private static string server = "localhost";
	private static string path = "";
	private static string Page = "";
	private static AsyncData ad = new AsyncData();
	private static int BytesReceived = -1; // can't be 0

	private static ManualResetEvent ConnectDone = new ManualResetEvent(false);
	private static ManualResetEvent SendDone = new ManualResetEvent(false);
	private static ManualResetEvent ReceiveDone = new ManualResetEvent(false);
	
	public static void Main (string [] args) 
	{        
		if (args.Length < 1)
		{
			Console.WriteLine("No server name entered. Default is localhost.");
		}
		else
			server = args[0];
				        
		DoGet();        
	}

	public static Socket GetSocket(string host, int port) 
	{
		Socket s = null;
		IPHostEntry iphe = Dns.Resolve(host);
		// Loop through all addresses not just
		// the first address. 
		foreach(IPAddress ipa in iphe.AddressList) 
		{
			IPEndPoint ipe = new IPEndPoint(ipa, port);
		
			// Use AddressFamily the AddressFamily that
			// is part of the endpoint. 
			// AddressFamily.InterNetwork limits you to
			// just IPv4
			Socket ts = new Socket(ipe.AddressFamily,
				SocketType.Stream,
				ProtocolType.Tcp);
			try 
			{
				// Return the first socket that can be
				// we can connect to.
				ts.Connect(ipe);
				s = ts;
				break;
			} 
			catch (ArgumentNullException ae)
			{
				Console.WriteLine("ArgumentNullException: " + ae.ToString());
			}
			catch (SocketException se)
			{				
				Console.WriteLine("SocketException: " + se.ToString());
			}
			catch (Exception e) 
			{
				Console.WriteLine("Connection failed: " + e.ToString());
			}
		}
		return s;
	}

	private static void DoGet()
	{
		try
		{
            Socket s = GetSocket(server, 80);

			s.Blocking = false; // set this to true (default) if you like

			// Set the send and receive timeouts to 10 seconds			
			s.SetSocketOption(SocketOptionLevel.Socket, 
				SocketOptionName.SendTimeout,
				10000);

			s.SetSocketOption(SocketOptionLevel.Socket, 
				SocketOptionName.ReceiveTimeout,
				10000);

			DoSend(s);
			SendDone.WaitOne();
			                			
			DoReceive(s);		
            
			Console.WriteLine("\nDefault HTML page on " + server + " received\n");
			Console.WriteLine(Page);			

			s.Shutdown(SocketShutdown.Both);
			s.Close();
		}

		catch (Exception e)
		{
			Console.WriteLine("Exception: " + e.Message);
			Console.WriteLine("StackTrace: " + e.StackTrace);
			return;
		}
	}

	private static void DoSend(Socket s) 
	{				
		Encoding ASCII = Encoding.ASCII;
		string Get = "GET /" + path + " HTTP/1.1\r\nHost: " + server + "\r\nConnection: Close\r\n\r\n";
		Byte[] ByteGet = ASCII.GetBytes(Get);            

		IAsyncResult result = s.BeginSend(ByteGet,                
			0,
			ByteGet.Length, 
			0,
			new AsyncCallback(SendCallback),
			s);        			
        
		return;		
	}

	private static void SendCallback(IAsyncResult result) 
	{        
		try
		{

			Socket socket = (Socket) result.AsyncState;

			int sentBytes = socket.EndSend(result);
			Console.WriteLine("\nSent " + sentBytes + " bytes to " + server);
        
			SendDone.Set();
			return;
		}

		catch (SocketException e)
		{
			// Send failed, do whatever you want here

			Console.WriteLine("Can't send data to: " + server);
			Console.WriteLine("\nSocketException: " + e.Message);
			Console.WriteLine("Error Code: " + e.NativeErrorCode);
			Console.WriteLine("StackTrace: " + e.StackTrace);	

			return;		
		}
	}

	private static void DoReceive(Socket s)
	{				
		try
		{
			ad.socket = s;

			while (BytesReceived != 0)
			{
				ReceiveDone.Reset();


				IAsyncResult result = s.BeginReceive(ad.RecBytes,
					0,
					ad.RecBytes.Length,
					SocketFlags.None,
					new AsyncCallback(ReceiveCallback),
					null); 

				ReceiveDone.WaitOne();        
			}		
                
			ReceiveDone.Set();
			return;
		}

		catch (SocketException e)
		{
			// Receive failed, do whatever you want here

			Console.WriteLine("Error receiving data from: " + server);
			Console.WriteLine("\nSocketException: " + e.Message);
			Console.WriteLine("Error Code: " + e.NativeErrorCode);
			Console.WriteLine("StackTrace: " + e.StackTrace);	

			return;
		}
	}

	private static void ReceiveCallback(IAsyncResult result) 
	{
		try
		{    
			Encoding ASCII = Encoding.ASCII;
			BytesReceived = ad.socket.EndReceive(result);							

			Page += ASCII.GetString(ad.RecBytes, 0, BytesReceived);
						
			ReceiveDone.Set();
			return;
		}

		catch (SocketException e)
		{
			// Receive failed, do whatever you want here

			Console.WriteLine("Error receiving data from: " + server);
			Console.WriteLine("\nSocketException: " + e.Message);
			Console.WriteLine("Error Code: " + e.NativeErrorCode);
			Console.WriteLine("StackTrace: " + e.StackTrace);	

			return;		
		}
	}

} // SockGet
